# PredGO

Predicting gene ontology functions of proteins through multi-level feature fusion.


## WebServer

http://predgo.denglab.org/

The web service provides protein function prediction from sequence and structure, and allows browsing of function predictions for over 200,000 human proteins predicted by PredGO.

## Dependencies

**PredGO** is tested to work under Python 3.8.

To install all dependencies run:

```
conda env create -f predgo.yaml
```

Activate the conda virtual environment

```
conda activate predgo
```

## Data

The dataset used is described in detail in the paper. The structure of the AlphaFold prediction used in the dataset and the trained model can be downloaded from the following address.

tip: 2016-MMseqs is an alias for Uniprot-GOA

[Download from webserver](http://predgo.denglab.org/help.html#dataset):

|             | Dataset                                                      | Structures of AlphaFold2 prediction                          | Trained models                                               |
| ----------- | ------------------------------------------------------------ | ------------------------------------------------------------ | ------------------------------------------------------------ |
| CAFA3       | [dataset_cafa3](http://predgo.denglab.org/html/examples/publicdata/dataset_cafa3.tar.gz) | [afdb_dir](http://predgo.denglab.org/html/examples/publicdata/afdb_dir_cafa3.tar.gz) | [trained_models](http://predgo.denglab.org/html/examples/publicdata/trained_models_CAFA3.tar.gz) |
| 2016-MMseqs | [dataset_2016](http://predgo.denglab.org/html/examples/publicdata/dataset_2016_mmseqs.tar.gz) | [afdb_dir](http://predgo.denglab.org/html/examples/publicdata/afdb_dir_2016_mmseqs.tar.gz) | [trained_models](http://predgo.denglab.org/html/examples/publicdata/trained_models_2016_MMseqs.tar.gz) |



Download from google drive:

|             | Dataset                                                      | Structures of AlphaFold2 prediction                          | Trained models                                               |
| ----------- | ------------------------------------------------------------ | ------------------------------------------------------------ | ------------------------------------------------------------ |
| CAFA3       | [dataset_cafa3](https://drive.google.com/file/d/109Nm0VM2ooMsKakJL0mQKB5XMgP043VB/view?usp=sharing) | [afdb_dir](https://drive.google.com/file/d/1lUJWTQZYzp_iwO5VG_1HM-QzG4RSXXAf/view?usp=sharing) | [trained_models](https://drive.google.com/file/d/16uN_-0LbozWQnFwifunKfoDPRwwDke3g/view?usp=sharing) |
| 2016-MMseqs | [dataset_2016](https://drive.google.com/file/d/1n1ZkdiJy_YG3jb0CPlojWiAzP94ptpuI/view) | [afdb_dir](https://drive.google.com/file/d/1QfWHRPsP57lyHZ_6M512WmQGlfRF55xX/view?usp=sharing) | [trained_models](https://drive.google.com/file/d/12dScUMWds2yS34t3BapgNOBA4gsUtCv4/view?usp=sharing) |

## Project file structure

```
predgo
├── checkpoint_dir
│   ├── PredGOModel_2016
│   └── PredGOModel_cafa3
├── data
│   ├── 2016-MMseqs
│   │   ├── afdb_dir
│   │   ├── atest_diamond.tsv
│   │   ├── binary_dir
│   │   ├── esm_dir
│   │   ├── go.obo
│   │   ├── ppi_score.tsv
│   │   ├── ppi_seqs.fasta
│   │   ├── PredGODataset
│   │   ├── terms-50.tsv
│   │   ├── test_seqs.fasta
│   │   ├── test.tsv
│   │   ├── train_seqs.fasta
│   │   ├── train.tsv
│   │   ├── validation_seqs.fasta
│   │   └── validation.tsv
│   ├── CAFA3
│   │   ├── afdb_dir
│   │   ├── esm_dir
│   │   ├── go.obo
│   │   ├── ppi_score.tsv
│   │   ├── ppi_seqs.fasta
│   │   ├── PredGODataset
│   │   ├── terms.tsv
│   │   ├── test_data_standardAA_1000.fasta
│   │   ├── test_data_standardAA_1000.tsv
│   │   ├── train_data_train_standardAA_1000_afdb.fasta
│   │   ├── train_data_train_standardAA_1000_afdb.tsv
│   │   ├── train_data_valid_standardAA_1000_afdb.fasta
│   │   └── train_data_valid_standardAA_1000_afdb.tsv
│   └── example
│       ├── binary_dir
│       ├── result
│       └── struct_dir
├── esm
│   ├── axial_attention.py
│   ├── constants.py
│   ├── data.py
│   ├── extract.py
│   ├── __init__.py
│   ├── model.py
│   ├── modules.py
│   ├── multihead_attention.py
│   ├── pretrained.py
│   └── version.py
├── evaluate_2016_PredGO.py
├── evaluate_cafa3_PredGO.py
├── extract_esm_features.py
├── gvp
│   ├── __init__.py
├── logs
├── predgo
│   ├── data.py
│   ├── __init__.py
│   ├── model.py
│   ├── modules.py
├── predgo.yaml
├── predict_PredGOModel_from_binary_dir.py
├── predict_PredGOModel_from_binary_file.py
├── predict_PredGOModel_from_pdb_file.py
├── readme.md
├── tools
│   ├── __init__.py
│   ├── log.py
│   ├── metrics.py
│   ├── protein_data_builder.py
│   ├── structure_data_parser.py
│   └── tools.py
├── trained_models
│   ├── 2016-MMseqs
│   ├── CAFA3
├── train_PredGOModel_2016.py
├── train_PredGOModel_cafa3.py
```


## Protein function prediction

Because of the complexity of the AlphaFold2 installation process and the large computational and storage resources required for the prediction structure, the repository does not provide code for protein prediction functions from sequences. To install AlphaFold2, go to [AlphaFold2](https://github.com/deepmind/alphafold). The need to predict protein function from sequence can be accessed by [PredGO web server](http://predgo.denglab.org/).

### Option 1: predicting functions of a protein from its PDB file

```
python predict_PredGOModel_from_pdb_file.py
```

### Option 2: predicting functions of a protein from its PDB dir

```
python predict_PredGOModel_from_pdb_dir.py
```

### Option 3: predicting functions of a protein from its binary file

```
python predict_PredGOModel_from_binary_file.py
```

### Option 4: predicting functions of a protein from its binary dir

```
python predict_PredGOModel_from_binary_dir.py
```

### Output:

```
[
  {
    "term": "GO:0008146",
    "score": 0.9979,
    "name": "sulfotransferase activity",
    "type": "molecular_function"
  },
  {
    "term": "GO:0003824",
    "score": 0.9943,
    "name": "catalytic activity",
    "type": "molecular_function"
  },
  {
    "term": "GO:0016782",
    "score": 0.9937,
    "name": "transferase activity, transferring sulfur-containing groups",
    "type": "molecular_function"
  },
  {
    "term": "GO:0016740",
    "score": 0.9654,
    "name": "transferase activity",
    "type": "molecular_function"
  }
]
```

## Training PredGO

Training models with CAFA3 dataset

```
# Training the MF model
python train_PredGOModel_cafa3.py --ont mf --device cuda:0 --num_epochs 15
# Training the BP model
python train_PredGOModel_cafa3.py --ont bp--device cuda:0 --num_epochs 15
# Training the CC model
python train_PredGOModel_cafa3.py --ont cc --device cuda:0 --num_epochs 15
```

Training models with 2016-MMseqs dataset

```
# Training the MF model
python train_PredGOModel_2016.py --ont mf --device cuda:0 --num_epochs 15
# Training the BP model
python train_PredGOModel_2016.py --ont bp --device cuda:0 --num_epochs 15
# Training the CC model
python train_PredGOModel_2016.py --ont cc --device cuda:0 --num_epochs 15
```

The program automatically calls ESM-1b during training to generate features and save them in the folder esm_dir. If the generation fails you can try to call **extract_esm_featurs.py** manually to generate ESM-1b features.(ESM-1b features need to be generated for the sequences in the training set, validation set, test set and PPI set seqences)

```
# for example
python extract_esm_features.py esm1b_t33_650M_UR50S data/2016-MMseqs/train_seqs.fasta data/2016-MMseqs/esm_dir --repr_layers 33 --include mean
```



### Output

The trained models will be saved in the checkpoint directory, where the best performing models are saved in the file
starting with best_model, and the training results for each epoch will be saved.

Examples are as follows:

```
best_model_7.pth
ckpt_0.pth
ckpt_1.pth
ckpt_2.pth
ckpt_3.pth
ckpt_4.pth
ckpt_5.pth
ckpt_6.pth
ckpt_7.pth
ckpt_8.pth
ckpt_9.pth
```

## Citing

```
@article {predgo2022,
	author = {Zheng, Rongtao and Huang, zhijian and Deng, lei},
	title = {PredGO: Predicting gene ontology functions of proteins through multi-level feature fusion},
	year = {2022},
	doi = {},
	publisher = {},
	URL = {},
	journal = {}
}

```

